use strictures 1;
use Test::More;
use SCSite::PageSet;
use IO::All;

sub check_structure {
  my ($s) = @_;
  $s = { %$s };
  my $body = delete $s->{body};
  like($body, qr/^\s+<p>Some markdown here.<\/p>\s+$/sm, 'Body correct');
  delete @{$s}{grep /^_/, keys %$s};
  delete $s->{path};
  is_deeply($s,
    {
      created => '',
      title => '--TITLE--',
      description => '--DESCRIPTION--',
      keywords => '--KW1-- --KW2--'
    },
    'Metadata ok'
  );
}

my $ps = SCSite::PageSet->new(base_dir => io->dir('t/pages'));

my $test_html = q{<html>
  <head>
    <title>--TITLE--</title>
    <meta name="description" content="--DESCRIPTION--" />
    <meta name="keywords" content="--KW1-- --KW2--" />
  </head>
  <body>
    <p>Some markdown here.</p>
  </body>
</html>};

check_structure($ps->_extract_from_html($test_html));

my $test_md = q{Title: --TITLE--
description: --DESCRIPTION--
keywords: --KW1-- --KW2--

Some markdown here.
};

check_structure($ps->_extract_from_md($test_md));

check_structure($ps->get({ path => 'index' }));

check_structure($ps->get({ path => 'one/two' }));

done_testing;
